@@ -25,6 +25,22 @@
 
 #include "common.h"
 
+void prefetched_sign_algo_init(void);
+
+#ifdef HAS_3_0_API
+enum pkey_format_t {
+    PKEY_PUB  = 0,
+    PKEY_PRIV = 1,
+    PKEY_PRIV_SEED = 2
+};
+int get_pkey_from_octet_string(ErlNifEnv*,
+                               ERL_NIF_TERM alg_atom,
+                               ERL_NIF_TERM key_bin,
+                               enum pkey_format_t,
+                               EVP_PKEY **pkey_p,
+                               ERL_NIF_TERM *ret_p);
+#endif
+
 ERL_NIF_TERM pkey_sign_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[]);
 ERL_NIF_TERM pkey_verify_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[]);
 ERL_NIF_TERM pkey_crypt_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[]);
