@@ -39,7 +39,8 @@
 	 recv/2, recv/3,
 	 send/2,
 	 shutdown/2,
-	 sockname/1
+	 sockname/1,
+         info/1
 	]).
 
 
@@ -387,6 +388,10 @@ sockname(#{sock := Sock}) ->
     end.
 
 
+info(#{sock := Sock}) ->
+    socket:info(Sock).
+
+
 %% ==========================================================================
 
 reader_init(ControllingProcess, Sock, Async, Active, Method) 
